import math
import cmath
import matplotlib.pyplot as plt
cimport numpy as np
import numpy as np

def create_jetcolorlist(jet_vals, int max_iter, reverse=False):
  #jetcmap = plt.cm.get_cmap("cool", max_iter) #generate a map violet/purple/blue
  jetcmap = plt.cm.get_cmap("CMRmap", max_iter) #generate a map yellow/red to darkblue 
  #jetcmap = plt.cm.get_cmap("YlGnBu", max_iter) #generate a yellow to blue map 
  jet_vals = jetcmap(np.arange(max_iter)) #extract those values as an array 
  if reverse:
    return jet_vals[::-1]
  else:
    return jet_vals

def pick_jetcolor(jet_vals, int max_iter, int i):
  x = jet_vals[i % max_iter]
  #print x
  return x

def mapcolor(int i, double r, c, int max_iter, jet_vals):
  di = float(i)
  zn = cmath.sqrt(r + c);
  hue = di - cmath.log(cmath.log(np.abs(zn.real))) / cmath.log(2.0)  # 2 is escape radius
  hue = hue.real
  hue = 0.95 + 20.0 * hue
  xx = pick_jetcolor(jet_vals,max_iter,int(hue))
  return xx

def mandel(double x, double y, int max_iter, jet_vals):
  cdef complex c = complex(x, y)
  cdef complex z = 0.0j
  cdef int i
  for i in range(max_iter):
    z = z*z + c
    re = z.real*z.real 
    im = z.imag*z.imag
    if (re+im) >= 1024.0:
      zz = [ij for ij in mapcolor(i, re, im, max_iter, jet_vals)]
      return zz
  return (0,0,0,1)

def create_fractal(min_x, max_x, min_y, max_y, image, iters, jet_vals):
  height = image.shape[0]
  width = image.shape[1]

  pixel_size_x = (max_x - min_x) / width
  pixel_size_y = (max_y - min_y) / height
    
  for x in range(width):
    real = min_x + x * pixel_size_x
    for y in range(height):
      imag = min_y + y * pixel_size_y
      color = mandel(real, imag, iters, jet_vals)
      image[y, x] =  color
  return image
